
set echo on

CREATE TYPE typ_obj_breaksum AS OBJECT
(
  sum    NUMBER,

  STATIC FUNCTION ODCIAggregateInitialize (
                  sctx IN OUT typ_obj_breaksum
                  ) RETURN NUMBER,

  MEMBER FUNCTION ODCIAggregateIterate (
                  self  IN OUT typ_obj_breaksum, 
                  value IN     NUMBER
                  ) RETURN NUMBER,

  MEMBER FUNCTION ODCIAggregateTerminate (
                  self   IN  typ_obj_breaksum, 
                  retval OUT NUMBER,
                  flags  IN  NUMBER
                  ) RETURN NUMBER,

  MEMBER FUNCTION ODCIAggregateMerge (
                  self IN OUT typ_obj_breaksum, 
                  ctx2 IN     typ_obj_breaksum
                  ) RETURN NUMBER
);
/

CREATE TYPE BODY typ_obj_breaksum IS 

   STATIC FUNCTION ODCIAggregateInitialize (
                   sctx IN OUT typ_obj_breaksum
                   ) RETURN NUMBER IS 
   BEGIN
     sctx := typ_obj_breaksum(0);
     RETURN ODCIConst.Success;
   END;

   MEMBER FUNCTION ODCIAggregateIterate (
                   self  IN OUT typ_obj_breaksum,
                   value IN     NUMBER
                   ) RETURN NUMBER IS
   BEGIN
     self.sum := CASE
                    WHEN value >= 0
                    OR  (value < 0 AND self.sum + value > 0)
                    THEN self.sum + value
                    ELSE 0
                 END;
     RETURN ODCIConst.Success;
   END;

   MEMBER FUNCTION ODCIAggregateTerminate (
                   self   IN  typ_obj_breaksum,
                   retval OUT NUMBER,
                   flags  IN  NUMBER
                   ) RETURN NUMBER IS
   BEGIN
     retval := self.sum;
     RETURN ODCIConst.Success;
   END;

   MEMBER FUNCTION ODCIAggregateMerge (
                   self IN OUT typ_obj_breaksum, 
                   ctx2 IN     typ_obj_breaksum
                   ) RETURN NUMBER IS
   BEGIN
     self.sum := CASE
                    WHEN self.sum + ctx2.sum > 0
                    THEN self.sum + ctx2.sum
                    ELSE 0
                 END;
     RETURN ODCIConst.Success;
   END;

END;
/
sho err

CREATE FUNCTION breaksum (input NUMBER) RETURN NUMBER 
   PARALLEL_ENABLE
   AGGREGATE USING typ_obj_breaksum;
/

